﻿Imports System.Collections


'#################################### Partie Méthodes de calcul #################################################

Partial Public Class SimulationDiffusionThermique2D


    ''' <summary>Initialise le composant avec les images et les paramètres</summary> 
    ''' <param name="imgConditionsInitiales">Bitmap contenant les températures à l'instant initial</param>
    ''' <param name="imgConditionsLimites">Bitmap contenant les conditions aux limites</param>
    ''' <param name="TemperatureMaxi">Température correspondant au rouge pur dans les bitmaps</param>
    ''' <param name="TemperatureMini">Température correspondant au noir dans les bitmaps</param>
    Sub Initialiser(ByVal imgConditionsInitiales As Bitmap, ByVal imgConditionsLimites As Bitmap, ByVal TemperatureMini As Double, ByVal TemperatureMaxi As Double)

        Reset()


        'Teste la présence d'erreurs
        If imgConditionsInitiales.Size <> imgConditionsLimites.Size Then Throw New Exception("Les images ne sont pas de même taille") : Exit Sub
        If TemperatureMaxi <= TemperatureMini Then Throw New Exception("La température mini est supérieure à la température maxi") : Exit Sub


        'Initialise les tableaux
        pLargeurTableau = imgConditionsInitiales.Width
        pHauteurTableau = imgConditionsInitiales.Height
        ReDim pTableau(pLargeurTableau - 1, pHauteurTableau - 1)
        ReDim pTableauConditionsLimites(pLargeurTableau - 1, pHauteurTableau - 1)

        pTempMini = TemperatureMini
        pTempMaxi = TemperatureMaxi



        '### Remplit le tableau des conditions initiales : ###

        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1

                'Pour chaque pixel, on met dans le tableau la température associée
                pTableau(x, y) = TemperatureMini + imgConditionsInitiales.GetPixel(x, y).R / 255 * (TemperatureMaxi - TemperatureMini)

            Next y
        Next x



        '### Remplit le tableau des conditions aux limites : ###

        'Teste chaque pixel de l'image (l'image des conditions aux limites bien sûr !)
        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1

                'Pixel blanc => pas de condition limite
                If imgConditionsLimites.GetPixel(x, y).R = 255 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 255 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLRien
                    GoTo skip
                End If

                'Pixel jaune => condition aux limites variable
                If imgConditionsLimites.GetPixel(x, y).R = 255 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLTempVariable
                    GoTo skip
                End If

                'Pixel bleu => enceinte adiabatique (flux thermique nul càd grad(T)=0)
                If imgConditionsLimites.GetPixel(x, y).R = 0 And imgConditionsLimites.GetPixel(x, y).G = 0 And imgConditionsLimites.GetPixel(x, y).B = 255 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLAdiabatique
                    pPrésenceAdiabatique = True
                    GoTo skip
                End If

                'Pixel vert => flux thermique imposé
                If imgConditionsLimites.GetPixel(x, y).R = 0 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxImposé
                    pPrésenceFluxImposé = True
                    GoTo skip
                End If

                'Pixel gris => flux conducto-convectif
                If imgConditionsLimites.GetPixel(x, y).R = imgConditionsLimites.GetPixel(x, y).G And imgConditionsLimites.GetPixel(x, y).R = imgConditionsLimites.GetPixel(x, y).B And imgConditionsLimites.GetPixel(x, y).R <> 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxCondConv
                    pPrésenceFluxCondConv = True
                    GoTo skip
                End If

                'Sinon c'est une nuance de rouge => température fixe
                pTableauConditionsLimites(x, y) = TemperatureMini + imgConditionsLimites.GetPixel(x, y).R / 255 * (TemperatureMaxi - TemperatureMini)
                'Cette température fixe devient de facto une température initiale
                pTableau(x, y) = pTableauConditionsLimites(x, y)

skip:
            Next y
        Next x


        'Vérifie l'existence de conditions aux limites sur les bords
        For x As Integer = 0 To pLargeurTableau - 1
            If pTableauConditionsLimites(x, 0) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(x, pHauteurTableau - 1) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next x

        For y As Integer = 0 To pHauteurTableau - 1
            If pTableauConditionsLimites(0, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(pLargeurTableau - 1, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next y


        'Place le tableau initial dans l'historique
        pHistorique.Add(pTableau)


    End Sub

    ''' <summary>
    ''' Initialise le composant avec des tableaux contenant les conditions initiales et aux limites
    ''' </summary>
    ''' <param name="TableauConditionsInitiales">Tableau contenant les températures initiales</param>
    ''' <param name="TableauConditionsLimites">Tableau contenant les conditions aux limites</param>
    ''' <remarks></remarks>
    Sub Initialiser(ByVal TableauConditionsInitiales As Double(,), ByVal TableauConditionsLimites As Double(,))

        Reset()

        'Vérifie que les tableaux ont la même taille
        If TableauConditionsInitiales.GetUpperBound(0) <> TableauConditionsLimites.GetUpperBound(0) Or TableauConditionsInitiales.GetUpperBound(1) <> TableauConditionsLimites.GetUpperBound(1) Then Throw New Exception("Les tableaux ne sont pas de même taille") : Exit Sub



        'Initialise les tableaux
        pTableau = TableauConditionsInitiales
        pTableauConditionsLimites = TableauConditionsLimites

        pLargeurTableau = TableauConditionsInitiales.GetUpperBound(0) + 1
        pHauteurTableau = TableauConditionsInitiales.GetUpperBound(1) + 1

        'Détermine les températures extrêmes (utile uniquement pour les méthodes ObtenirImage)
        pTempMini = Min(TableauConditionsInitiales)
        pTempMaxi = Max(TableauConditionsInitiales)
        If pTempMini = pTempMaxi Then pTempMaxi += 100



        'Vérifie l'existence de conditions aux limites sur les bords
        For x As Integer = 0 To pLargeurTableau - 1
            If pTableauConditionsLimites(x, 0) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(x, pHauteurTableau - 1) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next x

        For y As Integer = 0 To pHauteurTableau - 1
            If pTableauConditionsLimites(0, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(pLargeurTableau - 1, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next y



        'Teste la présence des différentes conditions aux limites
        pPrésenceAdiabatique = False
        pPrésenceFluxImposé = False
        pPrésenceFluxCondConv = False

        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLAdiabatique Then pPrésenceAdiabatique = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxImposé Then pPrésenceFluxImposé = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxCondConv Then pPrésenceFluxCondConv = True
            Next y
        Next x


        'Place le tableau initial dans l'historique
        pHistorique.Add(pTableau)


    End Sub

    '''<summary>
    ''' Initialise PasSpatial, PasTemporel et K
    ''' avec la plus grande valeur du pas temporel pour que la solution reste convergente
    ''' et retourne cette valeur de PasTemporel
    '''</summary>
    Function InitialiserPasTemporelMaxi(ByVal _PasSpatial As Double, ByVal _K As Double) As Double

        PasSpatial = _PasSpatial
        K = _K

        PasTemporel = 0.24 * PasSpatial ^ 2 / K

        Return PasTemporel

    End Function

    ''' <summary>Réinitialise pour un éventuel nouveau calcul (n'affecte pas les CI ni les CL)</summary>
    ''' <remarks>Déjà appelée par la méthode Initialiser</remarks>
    Sub Reset()
        pHistorique.Clear()
        pNombreDePasEffectués = 0
    End Sub



    ''' <summary>Avance d'un pas égal au pas temporel</summary>
    Sub AvancerUnPas()

        'Tableau au pas suivant
        Dim nouveauTableau(pLargeurTableau - 1, pHauteurTableau - 1) As Double


        'Pour chaque position
        For i As Integer = 0 To pLargeurTableau - 1
            For j As Integer = 0 To pHauteurTableau - 1

                Select Case pTableauConditionsLimites(i, j)
                    Case ConstanteCL.CLRien  'Pas de condition : on fait le calcul
                        nouveauTableau(i, j) = _
                        (1 - 4 * K * PasTemporel / (PasSpatial * PasSpatial)) * pTableau(i, j) + _
                        K * PasTemporel / (PasSpatial * PasSpatial) * (pTableau(i + 1, j) + pTableau(i - 1, j) + pTableau(i, j + 1) + pTableau(i, j - 1))
                    Case ConstanteCL.CLTempVariable  'Condition aux limites variable : on remplace par la température de la zone "conditions aux limites variables"
                        nouveauTableau(i, j) = CLTempératureVariable
                    Case Else   'Condition aux limites (température constante ou flux)
                        nouveauTableau(i, j) = pTableauConditionsLimites(i, j)
                End Select

            Next j
        Next i



        'Nouvelle passe pour les pixels adiabatiques (si nécessaire)
        If pPrésenceAdiabatique = True Then

            'Pixel adiabatique => grad(T) = 0
            'On remplace donc la température à l'interface par celle d'à côté

            For i As Integer = 1 To pLargeurTableau - 2
                For j As Integer = 1 To pHauteurTableau - 2

                    'Si le pixel adiabatique est à gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i + 1, j)
                    'Si le pixel adiabatique est à droite
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i - 1, j)
                    'Si le pixel adiabatique est en haut
                    If pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i, j + 1)
                    'Si le pixel adiabatique est en bas
                    If pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i, j - 1)

                    'Si le pixel adiabatique est dans le coin supérieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLAdiabatique AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i + 1, j + 1)
                    'Si le pixel adiabatique est dans le coin inférieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLAdiabatique AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i + 1, j - 1)
                    'Si le pixel adiabatique est dans le coin inférieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLAdiabatique AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i - 1, j - 1)
                    'Si le pixel adiabatique est dans le coin supérieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLAdiabatique AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLAdiabatique Then nouveauTableau(i, j) = nouveauTableau(i - 1, j + 1)

                Next j
            Next i

        End If



        'Nouvelle passe pour le flux imposé (si nécessaire)
        If pPrésenceFluxImposé = True Then

            Dim TSup As Double = CLFluxImposé * PasSpatial / Lambda

            For i As Integer = 1 To pLargeurTableau - 2
                For j As Integer = 1 To pHauteurTableau - 2

                    'Si le pixel est à gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i + 1, j) + TSup
                    'Si le pixel est à droite
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i - 1, j) + TSup
                    'Si le pixel est en haut
                    If pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i, j + 1) + TSup
                    'Si le pixel est en bas
                    If pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i, j - 1) + TSup

                    'Si le pixel est dans le coin supérieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxImposé AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i + 1, j + 1) + 1.414 * TSup
                    'Si le pixel est dans le coin inférieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxImposé AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i + 1, j - 1) + 1.414 * TSup
                    'Si le pixel est dans le coin inférieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxImposé AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i - 1, j - 1) + 1.414 * TSup
                    'Si le pixel est dans le coin supérieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxImposé AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxImposé Then nouveauTableau(i, j) = nouveauTableau(i - 1, j + 1) + 1.414 * TSup

                Next j
            Next i

        End If



        'Nouvelle passe pour le flux conducto-convectif (si nécessaire)
        If pPrésenceFluxCondConv = True Then

            For i As Integer = 1 To pLargeurTableau - 2
                For j As Integer = 1 To pHauteurTableau - 2

                    'Si le pixel est à gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i + 1, j) + CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i + 1, j)) * PasSpatial / Lambda
                    'Si le pixel est à droite
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i - 1, j) + CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i - 1, j)) * PasSpatial / Lambda
                    'Si le pixel est en haut
                    If pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i, j + 1) + CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i, j + 1)) * PasSpatial / Lambda
                    'Si le pixel est en bas
                    If pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i, j - 1) + CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i, j - 1)) * PasSpatial / Lambda

                    'Si le pixel est dans le coin supérieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxCondConv AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i + 1, j + 1) + 1.414 * CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i + 1, j + 1)) * PasSpatial / Lambda
                    'Si le pixel est dans le coin inférieur gauche
                    If pTableauConditionsLimites(i - 1, j) = ConstanteCL.CLFluxCondConv AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i + 1, j - 1) + 1.414 * CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i + 1, j - 1)) * PasSpatial / Lambda
                    'Si le pixel est dans le coin inférieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxCondConv AndAlso pTableauConditionsLimites(i, j + 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i - 1, j - 1) + 1.414 * CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i - 1, j - 1)) * PasSpatial / Lambda
                    'Si le pixel est dans le coin supérieur droit
                    If pTableauConditionsLimites(i + 1, j) = ConstanteCL.CLFluxCondConv AndAlso pTableauConditionsLimites(i, j - 1) = ConstanteCL.CLFluxCondConv Then nouveauTableau(i, j) = nouveauTableau(i - 1, j + 1) + 1.414 * CLFluxCondConvh * (CLFluxCondConvText - nouveauTableau(i - 1, j + 1)) * PasSpatial / Lambda

                Next j
            Next i

        End If


        'Mémorise le tableau du pas suivant
        pTableau = nouveauTableau

        'puis le place dans l'historique
        pHistorique.Add(pTableau)

        pNombreDePasEffectués += 1

        'A cet instant, pHistorique(pNombreDePasEffectues) et pTableau sont identiques


        If pConserverHistorique = False AndAlso pNombreDePasEffectués >= 3 Then
            pHistorique(pNombreDePasEffectués - 2) = Nothing
        End If


    End Sub

    ''' <summary>Avance d'un nombre défini de pas temporels</summary>
    ''' <param name="NombreDePas">Nombre de pas temporels</param>
    Sub AvancerPlusieursPas(ByVal NombreDePas As Integer)
        If NombreDePas <= 0 Then Throw New Exception("Nombre de pas négatif") : Exit Sub

        For i As Integer = 1 To NombreDePas
            AvancerUnPas()
        Next
    End Sub


    ''' <summary>
    ''' Modifie les conditions aux limites
    ''' </summary>
    ''' <param name="imgConditionsLimites">Bitmap contenant les nouvelles conditions aux limites</param>
    ''' <param name="TemperatureMini">Température correspondant au noir dans le bitmap</param>
    ''' <param name="TemperatureMaxi">Température correspondant au rouge pur dans le bitmap</param>
    ''' <remarks></remarks>
    Sub ModifierConditionsLimites(ByVal imgConditionsLimites As Bitmap, ByVal TemperatureMini As Double, ByVal TemperatureMaxi As Double)

        'Teste la présence d'erreurs
        If imgConditionsLimites.Width <> pLargeurTableau Or imgConditionsLimites.Height <> pHauteurTableau Then Throw New Exception("Le Bitmap n'est pas de la bonne taille") : Exit Sub
        If TemperatureMaxi < TemperatureMini Then Throw New Exception("La température mini est supérieure à la température maxi") : Exit Sub



        '### Remplit le tableau des conditions aux limites : ###

        'Teste chaque pixel de l'image (l'image des conditions aux limites bien sûr !)
        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1

                'Pixel blanc => pas de condition limite
                If imgConditionsLimites.GetPixel(x, y).R = 255 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 255 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLRien
                    GoTo skip
                End If

                'Pixel jaune => condition aux limites variable
                If imgConditionsLimites.GetPixel(x, y).R = 255 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLTempVariable
                    GoTo skip
                End If

                'Pixel bleu => enceinte adiabatique (flux thermique nul càd grad(T)=0)
                If imgConditionsLimites.GetPixel(x, y).R = 0 And imgConditionsLimites.GetPixel(x, y).G = 0 And imgConditionsLimites.GetPixel(x, y).B = 255 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLAdiabatique
                    pPrésenceAdiabatique = True
                    GoTo skip
                End If

                'Pixel vert => flux thermique imposé
                If imgConditionsLimites.GetPixel(x, y).R = 0 And imgConditionsLimites.GetPixel(x, y).G = 255 And imgConditionsLimites.GetPixel(x, y).B = 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxImposé
                    pPrésenceFluxImposé = True
                    GoTo skip
                End If

                'Pixel gris => flux conducto-convectif
                If imgConditionsLimites.GetPixel(x, y).R = imgConditionsLimites.GetPixel(x, y).G And imgConditionsLimites.GetPixel(x, y).R = imgConditionsLimites.GetPixel(x, y).B And imgConditionsLimites.GetPixel(x, y).R <> 0 Then
                    pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxCondConv
                    pPrésenceFluxCondConv = True
                    GoTo skip
                End If

                'Sinon c'est une nuance de rouge => température fixe
                pTableauConditionsLimites(x, y) = TemperatureMini + imgConditionsLimites.GetPixel(x, y).R / 255 * (TemperatureMaxi - TemperatureMini)

skip:
            Next y
        Next x



        'Vérifie l'existence de conditions aux limites sur les bords
        For x As Integer = 0 To pLargeurTableau - 1
            If pTableauConditionsLimites(x, 0) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(x, pHauteurTableau - 1) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next x

        For y As Integer = 0 To pHauteurTableau - 1
            If pTableauConditionsLimites(0, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If pTableauConditionsLimites(pLargeurTableau - 1, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next y



        'Teste la présence des différentes conditions aux limites
        pPrésenceAdiabatique = False
        pPrésenceFluxImposé = False
        pPrésenceFluxCondConv = False

        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLAdiabatique Then pPrésenceAdiabatique = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxImposé Then pPrésenceFluxImposé = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxCondConv Then pPrésenceFluxCondConv = True
            Next y
        Next x


    End Sub

    ''' <summary>
    ''' Modifie les conditions aux limites
    ''' </summary>
    ''' <param name="TableauConditionsLimites">Tableau contenant les nouvelles conditions aux limites</param>
    ''' <remarks></remarks>
    Sub ModifierConditionsLimites(ByVal TableauConditionsLimites As Double(,))

        'Vérifie que le tableau a la bonne taille
        If TableauConditionsLimites.GetUpperBound(0) <> pTableauConditionsLimites.GetUpperBound(0) Or TableauConditionsLimites.GetUpperBound(1) <> pTableauConditionsLimites.GetUpperBound(1) Then Throw New Exception("Le tableau n'a pas la bonne taille")

        'Vérifie l'existence de conditions aux limites sur les bords
        For x As Integer = 0 To pLargeurTableau - 1
            If TableauConditionsLimites(x, 0) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If TableauConditionsLimites(x, pHauteurTableau - 1) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next x

        For y As Integer = 0 To pHauteurTableau - 1
            If TableauConditionsLimites(0, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
            If TableauConditionsLimites(pLargeurTableau - 1, y) = ConstanteCL.CLRien Then Throw New Exception("Conditions aux limites insuffisantes")
        Next y



        'Remplace le tableau des conditions aux limites
        pTableauConditionsLimites = TableauConditionsLimites


        'Teste la présence des différentes conditions aux limites
        pPrésenceAdiabatique = False
        pPrésenceFluxImposé = False
        pPrésenceFluxCondConv = False

        For x As Integer = 0 To pLargeurTableau - 1
            For y As Integer = 0 To pHauteurTableau - 1
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLAdiabatique Then pPrésenceAdiabatique = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxImposé Then pPrésenceFluxImposé = True
                If pTableauConditionsLimites(x, y) = ConstanteCL.CLFluxCondConv Then pPrésenceFluxCondConv = True
            Next y
        Next x


    End Sub



    'Fonction locale retournant la valeur minimum d'un tableau de Double
    Private Function Min(ByVal Tableau As Double(,)) As Double
        Dim temp As Double = Tableau(0, 0)

        For i As Integer = 0 To Tableau.GetUpperBound(0)
            For j As Integer = 0 To Tableau.GetUpperBound(1)
                If Tableau(i, j) < temp Then temp = Tableau(i, j)
            Next j
        Next i

        Return temp

    End Function

    'Idem mais valeur maxi
    Private Function Max(ByVal Tableau As Double(,)) As Double
        Dim temp As Double = Tableau(0, 0)

        For i As Integer = 0 To Tableau.GetUpperBound(0)
            For j As Integer = 0 To Tableau.GetUpperBound(1)
                If Tableau(i, j) > temp Then temp = Tableau(i, j)
            Next j
        Next i

        Return temp

    End Function

    'Fonction locale retournant la somme de deux tableaux de Double
    Private Function Somme(ByVal t1 As Double(,), ByVal t2 As Double(,)) As Double(,)
        Dim mSomme(t1.GetUpperBound(0), t1.GetUpperBound(1)) As Double
        For i As Integer = 0 To t1.GetUpperBound(0)
            For j As Integer = 0 To t1.GetUpperBound(1)
                mSomme(i, j) = t1(i, j) + t2(i, j)
            Next j
        Next i
        Return mSomme
    End Function

    'Idem mais différence
    Private Function Difference(ByVal t1 As Double(,), ByVal t2 As Double(,)) As Double(,)
        Dim mDiff(t1.GetUpperBound(0), t1.GetUpperBound(1)) As Double
        For i As Integer = 0 To t1.GetUpperBound(0)
            For j As Integer = 0 To t1.GetUpperBound(1)
                mDiff(i, j) = t1(i, j) - t2(i, j)
            Next j
        Next i
        Return mDiff
    End Function

    'Fonction locale retournant la valeur absolue d'un tableau
    Private Function Abs(ByVal Tableau As Double(,)) As Double(,)
        Dim mAbs(Tableau.GetUpperBound(0), Tableau.GetUpperBound(1)) As Double
        For i As Integer = 0 To Tableau.GetUpperBound(0)
            For j As Integer = 0 To Tableau.GetUpperBound(1)
                mAbs(i, j) = Math.Abs(Tableau(i, j))
            Next j
        Next i
        Return mAbs
    End Function


End Class