﻿Imports System.Collections


'#################################### Partie Méthodes de sortie #################################################

Partial Public Class SimulationDiffusionThermique2D


    '''<summary>Retourne le tableau des températures au dernier instant calculé</summary>
    Function ObtenirTableauFinal() As Double(,)
        Return pTableau
    End Function

    ''' <summary>
    ''' Retourne le tableau des températures après un nombre de pas temporels donné
    ''' </summary>
    ''' <param name="Pas">Nombre de pas temporels</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirTableau(ByVal Pas As Integer) As Double(,)

        If pConserverHistorique = False And Pas < pNombreDePasEffectués Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez accéder qu'au tableau final")
        If Pas > pNombreDePasEffectués Then Throw New Exception("Numéro du pas supérieur au nombre de pas déjà effectués")

        Return pHistorique(Pas)

    End Function



    ''' <summary>
    ''' Retourne une image affichant les températures au dernier instant calculé
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirImageFinale() As Bitmap
        Return ObtenirImage(pNombreDePasEffectués)
    End Function

    ''' <summary>
    ''' Retourne une image affichant les températures au dernier instant calculé
    ''' </summary>
    ''' <param name="TemperatureMini">Température minimale, en dessous de laquelle les couleurs sont identiques</param>
    ''' <param name="TemperatureMaxi">Température maximale, au dessus de laquelle toutes les couleurs sont identiques</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirImageFinale(ByVal TemperatureMini As Double, ByVal TemperatureMaxi As Double) As Bitmap
        Return ObtenirImage(pNombreDePasEffectués, TemperatureMini, TemperatureMaxi)
    End Function


    ''' <summary>
    ''' Retourne une image affichant les températures après un nombre de pas temporels donnés
    ''' </summary>
    ''' <param name="Pas">Nombre de pas temporels</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirImage(ByVal Pas As Integer) As Bitmap

        If pConserverHistorique = False And Pas < pNombreDePasEffectués Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez accéder qu'à l'image finale")

        Return ObtenirImage(Pas, pTempMini, pTempMaxi)

    End Function

    ''' <summary>
    ''' Retourne une image affichant les températures après un nombre de pas temporels donnés
    ''' </summary>
    ''' <param name="Pas">Nombre de pas temporels</param>
    ''' <param name="TemperatureMini">Température minimale, en dessous de laquelle les couleurs sont identiques</param>
    ''' <param name="TemperatureMaxi">Température maximale, au dessus de laquelle toutes les couleurs sont identiques</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirImage(ByVal Pas As Integer, ByVal TemperatureMini As Double, ByVal TemperatureMaxi As Double) As Bitmap

        If pConserverHistorique = False And Pas < pNombreDePasEffectués Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez accéder qu'à l'image finale")
        If Pas > pNombreDePasEffectués Then Throw New Exception("Numéro du pas supérieur au nombre de pas déjà effectués")

        'Image des températures
        Dim mImage As New Bitmap(pLargeurTableau, pHauteurTableau)
        'Tableau temporaire contenant le tableau des températures à l'instant considéré (évite d'aller le chercher à chaque fois dans pHistorique (plus lent) )
        Dim mTempTableau As Double(,) = pHistorique(Pas)

        'Pour chaque position
        For i As Integer = 0 To pLargeurTableau - 1
            For j As Integer = 0 To pHauteurTableau - 1

                'Définit la couleur du pixel (simple règle de proportionnalité)
                Select Case mTempTableau(i, j)
                    Case Is < TemperatureMini
                        mImage.SetPixel(i, j, pDégradéTableauCouleurs(0))
                    Case Is > TemperatureMaxi
                        mImage.SetPixel(i, j, pDégradéTableauCouleurs(99))
                    Case Else
                        mImage.SetPixel(i, j, pDégradéTableauCouleurs(CInt(Math.Floor(99 * (mTempTableau(i, j) - TemperatureMini) / (TemperatureMaxi - TemperatureMini)))))
                End Select

                'Si c'est un pixel d'une zone adiabatique
                If pTableauConditionsLimites(i, j) = ConstanteCL.CLAdiabatique Then mImage.SetPixel(i, j, pCouleurAdiabatique)
                'Si c'est un pixel d'un flux thermique fixé
                If pTableauConditionsLimites(i, j) = ConstanteCL.CLFluxImposé Then mImage.SetPixel(i, j, pCouleurFluxImposé)
                'Si c'est un pixel d'un flux thermique conducto-convectif
                If pTableauConditionsLimites(i, j) = ConstanteCL.CLFluxCondConv Then mImage.SetPixel(i, j, pCouleurFluxCondConv)

            Next j
        Next i

        'Retourne l'image des températures
        Return mImage

    End Function



    ''' <summary>
    ''' Retourne la température d'un point donné au dernier instant calculé
    ''' </summary>
    ''' <param name="x">Abscisse du point</param>
    ''' <param name="y">Ordonnée du point</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirTempératureFinale(ByVal x As Integer, ByVal y As Integer) As Double

        Return pHistorique(pNombreDePasEffectués)(x, y)

    End Function

    ''' <summary>
    ''' Retourne la température d'un point donné à un instant donné
    ''' </summary>
    ''' <param name="Pas">Nombre de pas temporels</param>
    ''' <param name="x">Abscisse du point</param>
    ''' <param name="y">Ordonnée du point</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirTempérature(ByVal Pas As Integer, ByVal x As Integer, ByVal y As Integer) As Double

        If pConserverHistorique = False And Pas < pNombreDePasEffectués Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez accéder qu'à la température finale")
        If Pas > pNombreDePasEffectués Then Throw New Exception("Numéro du pas supérieur au nombre de pas effectués")

        Return pHistorique(Pas)(x, y)

    End Function

    ''' <summary>
    ''' Retourne un tableau unidimensionnel contenant la température d'un point donné au cours du temps
    ''' </summary>
    ''' <param name="x">Abscisse du point</param>
    ''' <param name="y">Ordonnée du point</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirHistoriqueTempérature(ByVal x As Integer, ByVal y As Integer) As Double()
        If pConserverHistorique = False Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez pas obtenir la température en fonction du temps")
        Return ObtenirHistoriqueTempérature(x, y, 0, pNombreDePasEffectués)
    End Function

    ''' <summary>
    ''' Retourne un tableau unidimensionnel contenant la température d'un point donné au cours du temps entre deux instants donnés
    ''' </summary>
    ''' <param name="x">Abscisse du point</param>
    ''' <param name="y">Ordonnée du point</param>
    ''' <param name="PasInitial">Instant initial (nombre de pas temporels)</param>
    ''' <param name="PasFinal">Instant final (nombre de pas temporels)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function ObtenirHistoriqueTempérature(ByVal x As Integer, ByVal y As Integer, ByVal PasInitial As Integer, ByVal PasFinal As Integer) As Double()

        If pConserverHistorique = False Then Throw New Exception("ConserverHistorique est égal à False. Vous ne pouvez pas obtenir la température en fonction du temps")
        If PasInitial < 0 Or PasFinal > pNombreDePasEffectués Then Throw New Exception("Numéro de pas incorrect")
        If x < 0 Or x > pLargeurTableau Then Throw New Exception("Le point demandé n'existe pas")
        If y < 0 Or y > pHauteurTableau Then Throw New Exception("Le point demandé n'existe pas")

        'Tableau unidimensionnel contenant l'historique
        Dim mListe(PasFinal - PasInitial) As Double

        'On complète le tableau
        For i As Integer = PasInitial To PasFinal
            mListe(i) = pHistorique(i)(x, y)
        Next i

        'Et on renvoie le tableau.. simple non ??
        Return mListe

    End Function



    ''' <summary>
    ''' Retourne une image de 100 pixels par 1 pixel contenant le dégradé de températures en cours
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function DégradéObtenirImage() As Bitmap
        'Image qui va contenir le dégradé (100 pixels de large et 1 pixel de haut)
        Dim mImage As New Bitmap(100, 1)

        'Définit la couleur de chaque pixel
        For i As Integer = 0 To 99
            mImage.SetPixel(i, 0, pDégradéTableauCouleurs(i))
        Next i

        'Retourne l'image
        Return mImage

    End Function


End Class
