Imports System.Collections


'########################### Partie Variables, Proprits et Initialisation ##################################

Partial Public Class SimulationDiffusionThermique2D


#Region "Variables"

    ''' <summary>Temps entre deux pas (en s)</summary>
    Public PasTemporel As Double

    ''' <summary>Distance entre deux pixels (en m)</summary>
    Public PasSpatial As Double

    ''' <summary>Diffusivit thermique (en m/s)  [K = lambda / (rho . Cp)]</summary>
    Public K As Double

    ''' <summary>Conductivit  (en W/m/K)</summary>
    Public Lambda As Double

    ''' <summary>Temprature instantane des zones 'conditions aux limites variables' (en C)</summary>
    Public CLTempratureVariable As Double

    ''' <summary>Paramtre 'h' du flux conducto-convectif (en W/m/K)</summary>
    Public CLFluxCondConvh As Double

    ''' <summary>Paramtre 'Temprature extrieure' du flux conducto-convectif (en C)</summary>
    Public CLFluxCondConvText As Double

    ''' <summary>Valeur du flux impos (en W/m)</summary>
    Public CLFluxImpos As Double



    ''' <summary>
    ''' Enumre les diffrents types de dgrads
    ''' </summary>
    ''' <remarks></remarks>
    Enum CouleurDgrad As Integer
        TempraturesMulti = 0
        NoirEtRouge = 1
        NoirEtBlanc = 2
        BleuEtRouge = 3
        DgradPerso = 4
    End Enum

    ''' <summary>
    ''' Enumre les diffrents types de conditions aux limites
    ''' </summary>
    ''' <remarks></remarks>
    Enum ConstanteCL As Integer
        ''' <summary>Pas de conditions aux limites : diffusion thermique</summary>
        CLRien = -9990
        ''' <summary>Temprature variable</summary>
        CLTempVariable = -9991
        ''' <summary>Paroi adiabatique</summary>
        CLAdiabatique = -9992
        ''' <summary>Flux thermique impos</summary>
        CLFluxImpos = -9993
        ''' <summary>Flux conducto-convectif</summary>
        CLFluxCondConv = -9994

    End Enum




    'Variables locales (private)

    Dim pTableau(,) As Double       'Tableau du champ de tempratures EN COURS
    Dim pTableauConditionsLimites(,) As Double    'Tableau contenant les conditions aux limites
    Dim pLargeurTableau As Integer  'Largeur du tableau
    Dim pHauteurTableau As Integer  'Hauteur du tableau

    Dim pConserverHistorique As Boolean             'Indique s'in faut conserver en mmoire les tableaux intermdiaires
    Dim pHistorique As New Generic.List(Of Double(,))   'Liste contenant tous les tableaux dj calculs
    Dim pNombreDePasEffectus As Integer            'Nombre de pas dj effectus

    Dim pDgradTypeCouleurs As CouleurDgrad  'Type de dgrad pour l'image
    Dim pDgradPersoBitmap As Bitmap           'Bitmap contenant les couleurs du dgrad perso
    Dim pDgradCouleurFroid As Color           'Couleur correspondant  la temprature mini (pour les dgrads bicolores)
    Dim pDgradCouleurChaud As Color           'Couleur correspondant  la temprature maxi (pour les dgrads bicolores)
    Dim pDgradTableauCouleurs(99) As Color    'Tableau contenant les couleurs du dgrad
    Dim pCouleurAdiabatique As Color        'Couleur choisie pour afficher les zones adiabatiques
    Dim pCouleurFluxImpos As Color         'Couleur choisie pour afficher les zones de flux impos
    Dim pCouleurFluxCondConv As Color       'Couleur choisie pour afficher les zones de flux conducto-convectif

    Dim pTempMini As Double             'Temprature mini (temprature correspondant au noir pour un bitmap)
    Dim pTempMaxi As Double             'Temprature maxi (temprature correspondant au rouge pour un bitmap)

    Dim pPrsenceAdiabatique As Boolean = False     'Indique si des pixels adiabatiques sont prsents dans les conditions aux limites
    Dim pPrsenceFluxCondConv As Boolean = False    'Indique si un flux conducto-convectif est prsent dans les conditions aux limites
    Dim pPrsenceFluxImpos As Boolean = False      'Indique si un flux impos est prsent dans les conditions aux limites


#End Region


#Region "Proprits"


    ''' <summary>
    ''' Retourne les dimensions du tableau 
    ''' </summary>
    ReadOnly Property Taille() As Size
        Get
            Return New Size(pLargeurTableau, pHauteurTableau)
        End Get
    End Property

    ''' <summary>
    ''' Retourne le nombre de pas effectus
    ''' </summary>
    ReadOnly Property NombreDePasEffectus() As Integer
        Get
            Return pHistorique.Count - 1
        End Get
    End Property

    ''' <summary>
    ''' Indique si la solution converge
    ''' </summary>
    ReadOnly Property SolutionConvergente() As Boolean
        Get
            If K * PasTemporel / (PasSpatial ^ 2) < 0.25 Then
                Return True
            Else
                Return False
            End If
        End Get
    End Property

    ''' <summary>
    ''' Retourne l'cart maximal de temprature entre les 2 derniers pas
    ''' </summary>
    ReadOnly Property Prcision() As Double
        Get
            Return Max(Abs(Difference(pTableau, pHistorique(pNombreDePasEffectus - 1))))
            'Remarque : calcul long....
        End Get
    End Property



    ''' <summary>
    ''' Retourne la temprature mini (temprature correspondant au noir pour le bitmap initial)
    ''' </summary>
    Property TempratureMini() As Double
        Get
            Return pTempMini
        End Get
        Set(ByVal value As Double)
            pTempMini = value
        End Set
    End Property

    ''' <summary>
    ''' Retourne la temprature maxi (temprature correspondant au rouge pour le bitmap initial)
    ''' </summary>
    Property TempratureMaxi() As Double
        Get
            Return pTempMaxi
        End Get
        Set(ByVal value As Double)
            pTempMaxi = value
        End Set
    End Property



    ''' <summary>
    ''' Indique si le composant mmorise l'ensemble les tapes ou seulement l'tape finale
    ''' (recommand pour minimiser l'occupation mmoire)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property ConserverHistorique() As Boolean
        Get
            Return pConserverHistorique
        End Get
        Set(ByVal value As Boolean)
            pConserverHistorique = value
        End Set
    End Property



    ''' <summary>
    ''' Obtient ou dfinit le dgrad utilis pour les bitmaps
    ''' </summary>
    Property DgradType() As CouleurDgrad
        Get
            Return pDgradTypeCouleurs
        End Get

        Set(ByVal value As CouleurDgrad)

            pDgradTypeCouleurs = value


            'Dfinit les couleurs extrmes en fonction du dgrad choisi
            Select Case pDgradTypeCouleurs
                Case CouleurDgrad.BleuEtRouge
                    pDgradCouleurFroid = Color.Blue
                    pDgradCouleurChaud = Color.Red
                Case CouleurDgrad.NoirEtBlanc
                    pDgradCouleurFroid = Color.Black
                    pDgradCouleurChaud = Color.White
                Case CouleurDgrad.NoirEtRouge
                    pDgradCouleurFroid = Color.Black
                    pDgradCouleurChaud = Color.Red
            End Select

            'Cr la brosse de dgrad
            Dim Dimensions As New Rectangle(0, 0, 100, 1)
            Dim LinGradBrush As New Drawing2D.LinearGradientBrush(Dimensions, pDgradCouleurFroid, pDgradCouleurChaud, Drawing2D.LinearGradientMode.Horizontal)

            'Si le dgrad slectionn est TemperatureMulti, on dfinit les couleurs intermdiaires (blanc et jaune)
            If pDgradTypeCouleurs = CouleurDgrad.TempraturesMulti Then
                Dim Degrade As New Drawing2D.ColorBlend(4)
                Dim Couleurs As Color() = {Color.Blue, Color.FromArgb(80, 80, 255), Color.White, Color.Yellow, Color.Red}
                Dim Repartition() As Single = {0, 0.1, 0.5, 0.6, 1}
                Degrade.Colors = Couleurs
                Degrade.Positions = Repartition

                LinGradBrush.InterpolationColors = Degrade
                LinGradBrush.GammaCorrection = True
            End If

            'Cr un nouveau bitmap
            Dim Bp As New Bitmap(Dimensions.Width, Dimensions.Height)
            Dim Graph As Graphics = Graphics.FromImage(Bp)

            'Applique le dgrad dans le bitmap
            Graph.FillRectangle(LinGradBrush, Dimensions)


            '### Si dgrad perso, alors remplace le bitmap BP par le bitmap perso ###'
            If pDgradTypeCouleurs = CouleurDgrad.DgradPerso Then Bp = pDgradPersoBitmap
            If Bp Is Nothing Then Bp = New Bitmap(100, 1)


            'Enregistre chaque couleur du bitmap dans le tableau
            For i As Integer = 0 To 99
                pDgradTableauCouleurs(i) = Bp.GetPixel(i, 0)
            Next

        End Set
    End Property

    ''' <summary>
    ''' Obtient ou dfinit le bitmap utilis pour le dgrad perso
    ''' </summary>
    Property DgradPersoBitmap() As Bitmap
        Get
            If pDgradPersoBitmap Is Nothing Then Return New Bitmap(100, 1)
            Return pDgradPersoBitmap
        End Get
        Set(ByVal value As Bitmap)
            If value.Width <> 100 Then Throw New Exception("Le Bitmap contenant le dgrad personnalis doit mesurer 100 pixels de large") : Exit Property
            pDgradPersoBitmap = value

            'Permet de recalculer le tableau des couleurs
            If pDgradTypeCouleurs = CouleurDgrad.DgradPerso Then
                DgradType = CouleurDgrad.DgradPerso
            End If
        End Set
    End Property

    ''' <summary>Couleur d'affichage d'un pixel adiabatique</summary>
    Property CouleurAdiabatique() As Color
        Get
            Return pCouleurAdiabatique
        End Get
        Set(ByVal value As Color)
            pCouleurAdiabatique = value
        End Set
    End Property

    ''' <summary>Couleur d'affichage du flux impos</summary>
    Property CouleurFluxImpos() As Color
        Get
            Return pCouleurFluxImpos
        End Get
        Set(ByVal value As Color)
            pCouleurFluxImpos = value
        End Set
    End Property

    ''' <summary>Couleur d'affichage du flux conducto-convectif</summary>
    Property CouleurFluxCondConv() As Color
        Get
            Return pCouleurFluxCondConv
        End Get
        Set(ByVal value As Color)
            pCouleurFluxCondConv = value
        End Set
    End Property



#End Region


    'Lors de l'instanciation
    ''' <summary>
    ''' Instancie un objet permettant de rsoudre l'quation de la chaleur
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        'Proprits par dfaut :
        Me.DgradType = CouleurDgrad.TempraturesMulti
        Me.CouleurAdiabatique = Color.Black
        Me.CouleurFluxCondConv = Color.Purple
        Me.CouleurFluxImpos = Color.Purple
        Me.ConserverHistorique = True
    End Sub


End Class