﻿Imports ZedGraph

Public Class ListPlot


#Region "Variables locales"

    Dim myPane As GraphPane


    '***********************************************
    'Variables correspondant aux propriétés
    Dim couleurArrierePlan As Color = Color.White
    Dim grid As Boolean = False

    Dim titleText As String = ""

    Dim xAxisLabelText As String = ""
    Dim xAxisType As AxisType = AxisType.Linear

    Dim yAxisLabelText As String = ""
    Dim yAxisType As AxisType = AxisType.Linear

    Dim zoomAutoX As Boolean = True
    Dim zoomAutoY As Boolean = True
    Dim zoomXmin As Double = -10
    Dim zoomXmax As Double = 10
    Dim zoomYmin As Double = -10
    Dim zoomYmax As Double = 10

    Dim epaisseur As Single = 2


    '***********************************************
    'Liste des couleurs d'affichage des courbes
    Dim colorList As New List(Of Color)

    'Liste des courbes affichées
    Dim curveList As New List(Of LineItem)
    'Nombre total de courbes
    Dim curveNumber As Integer


#End Region



#Region "Propriétés"

    ''' <summary>
    ''' Couleur d'arrière plan du graphe
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _BackColor() As Color

        Get
            Return couleurArrierePlan
        End Get

        Set(ByVal value As Color)
            couleurArrierePlan = value

            Try
                myPane.Fill.Color = Color.FromKnownColor(KnownColor.Control)
                myPane.Chart.Fill.Type = FillType.Solid
                myPane.Chart.Fill.Color = couleurArrierePlan
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Titre du graphe (vide => pas de titre)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _TitleText() As String

        Get
            Return titleText
        End Get

        Set(ByVal value As String)
            titleText = value

            Try
                myPane.Title.Text = titleText
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Affichage du quadrillage
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _Grid() As Boolean
        Get
            Return grid
        End Get

        Set(ByVal value As Boolean)
            grid = value

            Try
                myPane.XAxis.MajorGrid.IsVisible = grid
                myPane.YAxis.MajorGrid.IsVisible = grid
                Me.Refresh()
            Catch ex As Exception
            End Try
        End Set

    End Property


    ''' <summary>
    ''' Largeur du tracé des courbes
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _Width() As Single

        Get
            Return epaisseur
        End Get

        Set(ByVal value As Single)
            epaisseur = value

            Try
                For i As Integer = 0 To curveNumber - 1
                    curveList(i).Line.Width = epaisseur
                Next i
                myPane.AxisChange()
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property


    ''' <summary>
    ''' Texte de l'axe des abscisses (vide => pas d'étiquette)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _XAxisLabelText() As String

        Get
            Return xAxisLabelText
        End Get

        Set(ByVal value As String)
            xAxisLabelText = value

            Try
                myPane.XAxis.Title.Text = xAxisLabelText
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Type d'axe (graduations et échelle)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _XAxisType() As AxisType

        Get
            Return xAxisType
        End Get

        Set(ByVal value As AxisType)
            xAxisType = value

            Try
                myPane.XAxis.Type = xAxisType
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set
    End Property


    ''' <summary>
    ''' Texte de l'axe des ordonnées (vide => pas d'étiquette)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _YAxisLabelText() As String

        Get
            Return yAxisLabelText
        End Get

        Set(ByVal value As String)
            yAxisLabelText = value

            Try
                myPane.YAxis.Title.Text = yAxisLabelText
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Type d'axe (graduations et échelle)
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _YAxisType() As AxisType

        Get
            Return yAxisType
        End Get

        Set(ByVal value As AxisType)
            yAxisType = value

            Try
                myPane.YAxis.Type = yAxisType
                Me.Refresh()
            Catch ex As Exception
            End Try

        End Set

    End Property


    ''' <summary>
    ''' Zoom automatique ou manuel suivant l'axe des abscisses
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomAutoX() As Boolean

        Get
            Return zoomAutoX
        End Get

        Set(ByVal value As Boolean)
            zoomAutoX = value

            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Zoom automatique ou manuel suivant l'axe des ordonnées
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomAutoY() As Boolean

        Get
            Return zoomAutoY
        End Get

        Set(ByVal value As Boolean)
            zoomAutoY = value

            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try

        End Set

    End Property

    ''' <summary>
    ''' Valeur personnalisée de zoom
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomXmin() As Double
        Get
            Return zoomXmin
        End Get

        Set(ByVal value As Double)
            zoomXmin = value
            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try
        End Set
    End Property

    ''' <summary>
    ''' Valeur personnalisée de zoom
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomXmax() As Double
        Get
            Return zoomXmax
        End Get

        Set(ByVal value As Double)
            zoomXmax = value
            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try
        End Set
    End Property

    ''' <summary>
    ''' Valeur personnalisée de zoom
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomYmin() As Double
        Get
            Return zoomYmin
        End Get

        Set(ByVal value As Double)
            zoomYmin = value
            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try
        End Set
    End Property

    ''' <summary>
    ''' Valeur personnalisée de zoom
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property _ZoomYmax() As Double
        Get
            Return zoomYmax
        End Get

        Set(ByVal value As Double)
            zoomYmax = value
            Try
                AppliquerZoom()
            Catch ex As Exception
            End Try
        End Set
    End Property


    ''' <summary>
    ''' Retourne le nombre total de courbes affichées
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    ReadOnly Property _Count() As Integer
        Get
            Return curveNumber
        End Get
    End Property


#End Region



#Region "Evénements"

    'Au chargement du contrôle...
    Private Sub ListPlot_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        myPane = _ZedGraph.GraphPane

        myPane.Border.IsVisible = False


        '#### Applique les propriétés ####
        _BackColor = _BackColor
        _Grid = _Grid
        _TitleText = _TitleText
        _Width = _Width

        _XAxisLabelText = _XAxisLabelText
        _XAxisType = _XAxisType

        _YAxisLabelText = _YAxisLabelText
        _YAxisType = _YAxisType

        _ZoomAutoX = _ZoomAutoX
        _ZoomAutoY = _ZoomAutoY
        _ZoomXmin = _ZoomXmin
        _ZoomXmax = _ZoomXmax
        _ZoomYmin = _ZoomYmin
        _ZoomYmax = _ZoomYmax


        'Définit les différentes couleurs pour le tracé des courbes
        colorList.Add(Color.DarkBlue)
        colorList.Add(Color.DarkRed)
        colorList.Add(Color.DarkGreen)
        colorList.Add(Color.DarkMagenta)
        colorList.Add(Color.DarkOrange)

    End Sub

#End Region



#Region "Méthodes"

    ''' <summary>
    ''' Ajoute une courbe au graphe
    ''' </summary>
    ''' <param name="List">Liste de valeurs (Integer) à tracer</param>
    ''' <param name="Name">(Optionnel) Nom de la courbe qui apparaît dans la légende</param>
    ''' <remarks></remarks>
    Sub _AddCurve(ByVal List() As Integer, Optional ByVal Name As String = "")
        'Incrémente le nombre de courbes
        curveNumber += 1

        'Créé la liste d'abscisses
        Dim listeX(List.GetUpperBound(0)) As Double
        For i As Integer = 0 To List.GetUpperBound(0)
            listeX(i) = i + 1
        Next i

        'Créé une liste d'ordonnées en Double
        Dim listeY(List.GetUpperBound(0)) As Double
        For i As Integer = 0 To List.GetUpperBound(0)
            listeY(i) = List(i)
        Next i


        'Définit la couleur de la courbe
        Dim col As Color
        Try
            col = colorList(curveNumber - 1)
        Catch ex As Exception
            col = Color.Black
        End Try


        'Trace la courbe
        Dim myCurve As LineItem = myPane.AddCurve(Name, listeX, listeY, col, SymbolType.None)
        'Modifie l'épaisseur de la courbe
        myCurve.Line.Width = 2

        'Ajoute la courbe à la liste
        curveList.Add(myCurve)


        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub

    ''' <summary>
    ''' Ajoute une courbe au graphe
    ''' </summary>
    ''' <param name="List">Liste de valeurs (Double) à tracer</param>
    ''' <param name="Name">(Optionnel) Nom de la courbe qui apparaît dans la légende</param>
    ''' <remarks></remarks>
    Sub _AddCurve(ByVal List() As Double, Optional ByVal Name As String = "")
        'Incrémente le nombre de courbes
        curveNumber += 1


        'Créé la liste d'abscisses
        Dim listeX(List.GetUpperBound(0)) As Double
        For i As Integer = 0 To List.GetUpperBound(0)
            listeX(i) = i + 1
        Next i


        'Définit la couleur de la courbe
        Dim col As Color
        Try
            col = colorList(curveNumber - 1)
        Catch ex As Exception
            col = Color.Black
        End Try


        'Trace la courbe
        Dim myCurve As LineItem = myPane.AddCurve(Name, listeX, List, col, SymbolType.None)
        'Modifie l'épaisseur de la courbe
        myCurve.Line.Width = epaisseur

        'Ajoute la courbe à la liste
        curveList.Add(myCurve)


        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub

    ''' <summary>
    ''' Ajoute une courbe au graphe
    ''' </summary>
    ''' <param name="ListX">Liste contenant les abscisses (Integer) des points à tracer</param>
    ''' <param name="ListY">Liste contenant les ordonnées (Integer) des points à tracer</param>
    ''' <param name="Name">(Optionnel) Nom de la courbe qui apparaît dans la légende</param>
    ''' <param name="Sort">(Optionnel) Si True, ordonne les points par abscisses croissantes</param>
    ''' <remarks></remarks>
    Sub _AddCurve(ByVal ListX() As Integer, ByVal ListY() As Integer, Optional ByVal Name As String = "", Optional ByVal Sort As Boolean = False)
        'Vérifie si les 2 listes contiennent le même nombre d'éléments
        If ListX.GetLength(0) <> ListY.GetLength(0) Then Throw New Exception("Taille des listes différentes") : Exit Sub

        'Incrémente le nombre de courbes
        curveNumber += 1

        'Nouvelles listes de Doubles
        Dim listeX2(ListX.GetUpperBound(0)) As Double
        Dim listeY2(ListX.GetUpperBound(0)) As Double

        For i As Integer = 0 To ListX.GetUpperBound(0)
            listeX2(i) = ListX(i)
            listeY2(i) = ListY(i)
        Next i

        'Trie éventuellement les points
        If Sort = True Then
            Trier(listeX2, listeY2)
        End If


        'Définit la couleur de la courbe
        Dim col As Color
        Try
            col = colorList(curveNumber - 1)
        Catch ex As Exception
            col = Color.Black
        End Try


        'Trace la courbe
        Dim myCurve As LineItem = myPane.AddCurve(Name, listeX2, listeY2, col, SymbolType.None)
        'Modifie l'épaisseur de la courbe
        myCurve.Line.Width = epaisseur

        'Ajoute la courbe à la liste
        curveList.Add(myCurve)


        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub

    ''' <summary>
    ''' Ajoute une courbe au graphe
    ''' </summary>
    ''' <param name="ListX">Liste contenant les abscisses (Double) des points à tracer</param>
    ''' <param name="ListY">Liste contenant les ordonnées (Double) des points à tracer</param>
    ''' <param name="Name">(Optionnel) Nom de la courbe qui apparaît dans la légende</param>
    ''' <param name="Sort">(Optionnel) Si True, ordonne les points par abscisses croissantes</param>
    ''' <remarks></remarks>
    Sub _AddCurve(ByVal ListX() As Double, ByVal ListY() As Double, Optional ByVal Name As String = "", Optional ByVal Sort As Boolean = False)
        'Vérifie si les 2 listes contiennent le même nombre d'éléments
        If ListX.GetLength(0) <> ListY.GetLength(0) Then Throw New Exception("Taille des listes différentes") : Exit Sub

        'Incrémente le nombre de courbes
        curveNumber += 1

        'Trie éventuellement les points
        If Sort = True Then
            Trier(ListX, ListY)
        End If


        'Définit la couleur de la courbe
        Dim col As Color
        Try
            col = colorList(curveNumber - 1)
        Catch ex As Exception
            col = Color.Black
        End Try


        'Trace la courbe
        Dim myCurve As LineItem = myPane.AddCurve(Name, ListX, ListY, col, SymbolType.None)
        'Modifie l'épaisseur de la courbe
        myCurve.Line.Width = epaisseur

        'Ajoute la courbe à la liste
        curveList.Add(myCurve)


        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub


    ''' <summary>
    ''' Supprime la courbe spécifiée
    ''' </summary>
    ''' <param name="Item">Index (de base 0) de la courbe à supprimer</param>
    ''' <param name="Update">(Optionnel) Si False, ne met pas jour l'affichage</param>
    ''' <remarks></remarks>
    Sub _RemoveCurve(ByVal Item As Integer, Optional ByVal Update As Boolean = True)
        'Vérifie que la courbe existe
        If Item >= curveNumber Then Throw New Exception("Courbe inexistante") : Exit Sub

        'Supprime la courbe de la liste
        curveList.RemoveAt(Item)
        'Supprime la courbe du graphe
        myPane.CurveList.RemoveAt(Item)
        'Décrémente le nombre de courbes
        curveNumber -= 1

        'Modifie la couleur des courbes restantes
        For i As Integer = 0 To curveNumber - 1
            Try
                curveList(i).Color = colorList(i)
            Catch ex As Exception
                curveList(i).Color = Color.Black
            End Try
        Next i

        'Rafraîchit éventuellement l'affichage
        If Update = True Then
            _ZedGraph.AxisChange()
            Me.Refresh()
        End If

    End Sub

    ''' <summary>
    ''' Supprime la dernière courbe ajoutée
    ''' </summary>
    ''' <param name="Update">(Optionnel) Si False, ne met pas jour l'affichage</param>
    ''' <remarks></remarks>
    Sub _RemoveLastCurve(Optional ByVal Update As Boolean = True)
        'Vérifie que la courbe existe
        If curveNumber = 0 Then Throw New Exception("Courbe inexistante") : Exit Sub
        'Efface la dernière courbe
        _RemoveCurve(curveNumber - 1, Update)
    End Sub


    ''' <summary>
    ''' Efface toutes les courbes du graphe
    ''' </summary>
    ''' <param name="Update">(Optionnel) Si False, ne met pas jour l'affichage</param>
    ''' <remarks></remarks>
    Sub _Clear(Optional ByVal Update As Boolean = True)
        'Efface tout
        curveList.Clear()
        myPane.CurveList.Clear()
        curveNumber = 0

        'Rafraîchit éventuellement l'affichage
        If Update = True Then
            _ZedGraph.AxisChange()
            Me.Refresh()
        End If

    End Sub


    ''' <summary>
    ''' Ajoute un point à la courbe spécifiée
    ''' </summary>
    ''' <param name="CurveItem">Index (de base 0) de la courbe utilisée</param>
    ''' <param name="Value">Ordonnée du point à ajouter (l'abscisse est automatiquement incrémentée de 1)</param>
    ''' <remarks></remarks>
    Sub _AddPoint(ByVal CurveItem As Integer, ByVal Value As Double)
        'Vérifie que la courbe existe
        If CurveItem >= curveNumber Then Throw New Exception("Courbe inexistante") : Exit Sub

        'Cherche l'abscisse maximmum
        Dim listeAbscisses As New List(Of Double)
        For i As Integer = 0 To curveList(CurveItem).Points.Count - 1
            listeAbscisses.Add(curveList(CurveItem).Points(i).X)
        Next
        listeAbscisses.Sort()
        Dim max As Double = listeAbscisses(curveList(CurveItem).Points.Count - 1)

        'Ajoute le point à la courbe
        curveList(CurveItem).AddPoint(max + 1, Value)

        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub

    ''' <summary>
    ''' Ajoute un point à la courbe spécifiée
    ''' </summary>
    ''' <param name="CurveItem">Index (de base 0) de la courbe utilisée</param>
    ''' <param name="X">Abscisse du point à ajouter</param>
    ''' <param name="Y">Ordonnée du point à ajouter</param>
    ''' <remarks></remarks>
    Sub _AddPoint(ByVal CurveItem As Integer, ByVal X As Double, ByVal Y As Double)
        'Vérifie que la courbe existe
        If CurveItem >= curveNumber Then Throw New Exception("Courbe inexistante") : Exit Sub

        'Ajoute le point à la courbe
        curveList(CurveItem).AddPoint(X, Y)

        'Rafraîchit l'affichage
        _ZedGraph.AxisChange()
        Me.Refresh()

    End Sub

#End Region



#Region "Méthodes locales"

    'Trie les points par abscisses croissantes
    Private Sub Trier(ByRef ListeX As Double(), ByRef ListeY As Double())

        '!!!!! Paramètres passés par référence : ils sont modifiés !!!!!


        'Créé une liste de PointF
        Dim listePoints(ListeX.GetUpperBound(0)) As PointF

        'Définit la valeur de chaque PointF
        For i As Integer = 0 To ListeX.GetUpperBound(0)
            listePoints(i).X = CSng(ListeX(i))
            listePoints(i).Y = CSng(ListeY(i))
        Next

        'Trie la liste de PointF en utilisant la liste d'abscisses
        Array.Sort(ListeX, listePoints)

        'Met le résultat dans les 2 listes
        For i As Integer = 0 To ListeX.GetUpperBound(0)
            ListeX(i) = listePoints(i).X
            ListeY(i) = listePoints(i).Y
        Next

    End Sub

    'Applique le zoom
    Private Sub AppliquerZoom()
        myPane.XAxis.Scale.Min = zoomXmin
        myPane.XAxis.Scale.Max = zoomXmax
        myPane.XAxis.Scale.MinAuto = zoomAutoX
        myPane.XAxis.Scale.MaxAuto = zoomAutoX

        myPane.YAxis.Scale.Min = zoomYmin
        myPane.YAxis.Scale.Max = zoomYmax
        myPane.YAxis.Scale.MinAuto = zoomAutoY
        myPane.YAxis.Scale.MaxAuto = zoomAutoY

        myPane.AxisChange()
        Me.Refresh()
    End Sub

#End Region


End Class
